import sys
import os
import os.path
import datetime, time
from time import sleep
import dateutil.parser

import subprocess

from PyQt5 import QtWidgets, QtGui, QtCore    #, uic, QtQml, QtQuick
from PyQt5.QtWidgets import QApplication, QScrollArea, QFileDialog, QMessageBox
from PyQt5.QtCore import Qt, QTimer

#from Dialog import Ui_Dialog as config_dialog

import configparser

import json

#import EchoReaderTwo
import EchoReader
#import EddsReader
import iws_defs

import HkMonitorUi
import TabsWindow
import PlotWindow
import MultiPlotWindow

import yaml

version = 'v2.1'

class HkMonitorWindow(QtWidgets.QMainWindow):
    
    def __init__(self, *, main=False, config, source, config_type):
        super(HkMonitorWindow, self).__init__()

        if main:
            print("Started by main")

        print("Configtype: " + str(config_type))

        self.setObjectName("MainWindow")
        self.setWindowTitle(f"IWS TMHK Monitor (Watcher) {version}")

        self.pause_hit = 0 
        self.watcher = QtCore.QFileSystemWatcher()

        self.file_time_seeker = {}
        self.acquire_dates_called = 0

        self.source = '' #os.environ['HOME'] + "/Projects/Euclid/DART_dumps/CEA_Day3_3.25.csv"
        self.source_selected = False
        self.mode = 'idle'

        self.event_queue = [False, False, False, False, False, False, False, False, False, False]

         # All uneeded. firsobt_done and currentobt_done replaced with first_obt and current_obt 
        #self.firstobt_done = None
        #self.lastobt_done = ''
        #self.acquireobt_done = ''
        #self.currentobt_done = ''

        self.currently_watching = False

        self.first_obt = None
        self.current_obt = None

        self.file_format = ''
        self.dump_dir = os.environ['HOME']

        self.config_type = config_type
        iws_defs.alter = True if self.config_type.startswith('EDDS') else False
        self.reader = EchoReader.EchoReader(config_type=config_type)
        #self.reader_out = EddsReader.EddsReader()

        # mcrw 20221012
        # Added this date conversion string for the TVAC 2022 tests. Conditional on config_type
        # 2022.281.16.11.57.112000
        print(f"HkMonitorWindow: init: config_type = {self.config_type}")
        #self.time_conv_str = "%Y.%j.%H.%M.%S.%f" if str(self.config_type) == "TVACC" else "%Y-%m-%dT%H:%M:%S.%f"
        #self.time_conv_str = "%Y.%j.%H.%M.%S.%f"
        self.time_conv_str = self.reader.tmpar_obt_conv
        ###print(f"HkMonitorWindow: init: time_conv_str = {self.time_conv_str}")


        self.output_lines = 0

        self._do_plot = 0
        #self.show()

        self.source = os.path.expanduser(source)
        self.source_selected = True
        self.config = config
        self.layout(iws_defs.defaults_directory + config, config_type, main)
        #self.start_watching()

        self.event_packet_count: UInt64 = 0
        self.tm_packet_count = 0
        self.tc_packet_count = 0
        self.hk_packet_count = 0

        self.rt_slow_count      = 0
        self.rt_fast_count      = 0
        self.rt_superfast_count = 0
        self.st_slow_count      = 0
        self.st_superfast_count = 0

        self.ready = True

        self.file_changed_count = 0
        self.dir_changed_count = 0

        self.monitors_used = {1 : False, 2 : False, 3 : False, 4 : False, 5 : False}

        """
        if len(args) == 2:
            self.source_selected = True
            self.source = os.path.expanduser(args[1])
            #self.read_ini()
            config = args[0]
            self.layout(iws_defs.defaults_directory + config)
            self.start_watching()

        else:
            self.open_dialog(iws_defs.config_file)
            config = iws_defs.config
            self.layout(iws_defs.defaults_directory + config)
            """


    def closeWindow(self):
        self.doQuit1()
        pass


    def request_hktm_metadata(self, connection, start_dt, end_dt):
        #print(f"request_hktm_metadata {connection}, {start_dt}, {end_dt}")
        now = datetime.datetime.utcnow()
        #print(f"now is {now}")
        if start_dt == '-1' and end_dt == '-1':
            self.realtime()
        elif start_dt != '-1' and end_dt == '-1':
            #print(f"Realtime with start time {start_dt} {now} - {start_dt}")
            pass
        elif start_dt != '-1' and end_dt != '-1':
            start_time = datetime.datetime.strptime(start_dt, "%Y-%m-%dT%H:%M:%S")
            end_time = datetime.datetime.strptime(end_dt, "%Y-%m-%dT%H:%M:%S")
            # s_diff is a timedelta object
            s_diff = now - start_time
            # e_diff is a timedelta object
            e_diff = now - end_time
            #print(f"Request {start_dt} {end_dt} ({s_diff}, {e_diff})")
        pass


    def start_button_clicked(self):
        sending_button = self.sender()
        if sending_button.accessibleName() == "START":
            self.ui.start_button.setText("Stop")
            self.ui.start_button.setAccessibleName("STOP")
            self.realtime()
        else:
            self.stop_acquisition()
            self.ui.start_button.setText("Start")
            self.ui.start_button.setAccessibleName("START")


    def realtime(self):
        #print("REALTIME")
        self.mode = 'realtime'
        self.ui.hk_monitor_mode.setText('Realtime')
        QtWidgets.QApplication.processEvents() # Last chance to allow gui updates before reading file
        self.start_watching()
        ###self.csv_update()

    def stop_acquisition(self):
        print("STOP ACQUISITION")
        self.mode = 'idle'
        self.ui.hk_monitor_mode.setText('Idle')
        self.stop_watching()


    # Create the ui
    def layout(self, config_file, config_type, main):
        self.ui = HkMonitorUi.HkMonitorUi(self)

        self.scroll = QScrollArea()             
        # Scroll Area which contains the widgets, set as the centralWidget
        #self.widget = QWidget()                 
        # Widget that contains the collection of Vertical Box

        #self.widget.setLayout(self.ui.layout(self,config_file))

        self.ui.layout(self, config_file, config_type, main)
        #self.setCentralWidget(self.ui)
        
        self.scroll.setVerticalScrollBarPolicy(Qt.ScrollBarAlwaysOn)
        self.scroll.setHorizontalScrollBarPolicy(Qt.ScrollBarAlwaysOn)
        self.scroll.setWidgetResizable(True)
        #self.scroll.setWidget(self.widget)
        self.scroll.setWidget(self.ui)
        self.setCentralWidget(self.scroll)

        self.ui.start_button.clicked.connect(self.start_button_clicked)
        self.ui.new_config_button.clicked.connect(self.new_config_button_clicked)
        self.ui.config_combo_box.currentIndexChanged.connect(self.config_combobox_clicked)
        #self.ui.new_plot_window_button.clicked.connect(self.new_plot_window_button_clicked)
        self.ui.open_window_button.clicked.connect(self.open_window_button_clicked)
        self.ui.open_plot_window_button.clicked.connect(self.open_plot_window_button_clicked)
        #self.ui.open_multi_plot_window_button.clicked.connect(self.open_multi_plot_window_button_clicked)
        self.ui.multi_plot_window_combobox.currentIndexChanged.connect(self.multi_plot_window_combox_clicked)
        self.ui.quit_button.clicked.connect(self.quit_button_clicked)

#        self.ui.top_frame_button_rewind.clicked.connect(self.rewind_clicked)
#        self.ui.top_frame_button_ff.clicked.connect(self.forward_clicked)
#        self.ui.top_frame_button_pause.clicked.connect(self.paused_clicked)
#        self.ui.top_frame_button_acquire.clicked.connect(self.acquire_clicked)

        self.ui.file_textbox.setText("Reading: " + self.source)

        self.show()

        
    def config_combobox_clicked(self, index):
        print(f'CHANGED: {index}')
        #if index != 0:
            #path = iws_defs.multi_plots_directory
            #filename = path + self.ui.multi_plot_window_combobox.itemText(index) + '.json'
            #window = MultiPlotWindow.MultiPlotWindow(filename, parent=self)
            #self.ui.plot_windows[window.name] = window
            #print(f'Opening multi-plot window {window.name} from {filename}')
        self.new_config_button_clicked()

    def new_config_button_clicked(self):
        return
        layout = self.ui.middle_frame.layout()
        count = self.ui.middle_frame.layout().count()
        #right_frame = self.ui.middle_frame.layout().itemAt(1)
        right_frame = layout.itemAt(1).widget()
        if right_frame != None:
            layout = right_frame.layout()
            count = layout.count()
            upper_frame = layout.itemAt(0).widget()
            lower_frame = layout.itemAt(1).widget()

            if upper_frame != None:
                layout = upper_frame.layout()
                count = layout.count()

                tab_widget = layout.itemAt(0).widget()
                if tab_widget != None:
                    layout = tab_widget.layout()
                    count = tab_widget.count()
                    
                    ###tab_widget.clear()

                    print(f"plots_tab_widget is {tab_widget}, layout {layout}, n tabs {count}")

                    self.ui.graph_frames.clear()

                    for index in range(count - 1, -1, -1):
                        print(f"Clearing plots {tab_widget.tabText(index)}, {tab_widget.widget(index)}")
                        tab_widget.widget(index).deleteLater()
                        tab_widget.removeTab(index)

            if lower_frame != None:
                layout = lower_frame.layout()
                count = layout.count()

                tab_widget = layout.itemAt(0).widget()
                if tab_widget != None:
                    layout = tab_widget.layout()
                    count = tab_widget.count()
                    
                    ###tab_widget.clear()

                    print(f"tab_widget is {tab_widget}, layout {layout}, n tabs {count}")

                    self.ui.label_frames.clear()

                    for index in range(count - 1, -1, -1):
                        print(f"Clearing {tab_widget.tabText(index)}, {tab_widget.widget(index)}")
                        tab_widget.widget(index).deleteLater()
                        tab_widget.removeTab(index)
                        
                
                #tab_frame = layout.itemAt(0).widget()
                #if tab_frame != None:
                    #######layout = lower_frame.layout()
                    #layout = tab_frame.layout()
                    #count = layout.count()
                    #print(f"tab_frame has {count} items")
                    #tab_widget = layout.itemAt(0).widget()
                    #if tab_widget != None:
                        #layout = tab_widget.layout()
                        #count = tab_widget.count()
                        #tab_widget.clear()
                        #print(f"tab_widget is {tab_widget}, layout {layout}, n tabs {count}")
        #count1 = right_frame.layout().count()
        #self.ui.middle_frame.right_frame.lower_frame.tab_frame.setStyleSheet("background-color:orange;") 

        print(f"new_config_button_clicked: len = {len(self.ui.label_frames)}, {count}, {right_frame}, {layout}")

        # Temporary
        #with open("/home/mcrw/Euclid/src/kevin/HKTM_monitor/config/defaults/systems_new_icd.json") as f:
        with open("/home/mcrw/IWSops_configurations/configurations/Big_window.json") as f:
            data = json.load(f)
            
        #self.ui.remake_upper_frame(data['graphs'])
        self.ui.remake_lower_frame(data['tabs'])

    
    def new_plot_window_button_clicked(self):
#        print(f"label frames = {self.ui.label_frames}")
        parameters = []
        for lf in self.ui.label_frames:
            for g in lf.groups:
                if g.selected:
                    g.debug()
                    parameters.append(g.get_labels())
                    #g.selected = False
                    g.reset_selected()
        print(f"open {self.source} '{self.first_obt}' '{self.current_obt}' '{parameters}'")
        pass
#        pid = subprocess.Popen(['python', './hk_playback.py', f"{config}", f"{self.current_catalog_file}", f"{self.start_datetime}", f"{self.end_datetime}"])


    def open_window_button_clicked(self):
        path = iws_defs.windows_directory
        options = QFileDialog.Options()
        options |= QFileDialog.DontUseNativeDialog
        suffixes = 'json file (*.json)'
        filename, _ = QFileDialog.getOpenFileName(self, "QFileDialog.getOpenFileName()", path, suffixes, options=options)
        
        if filename:
            window = TabsWindow.TabsWindow(filename, parent = self.ui)
            #print('Appending window')
            self.ui.windows[window.name] = window
            print(f"Opening window {window.name} from {filename}")


    # Open a seperate plot window
    def open_plot_window_button_clicked(self):
        path = iws_defs.plots_directory
        options = QFileDialog.Options()
        options |= QFileDialog.DontUseNativeDialog
        suffixes = 'json file (*.json)'
        filename, _ = QFileDialog.getOpenFileName(self, "QFileDialog.getOpenFileName()", path, suffixes, options=options)
        
        if filename:
            #print(f"Opening window {filename}")
            window = PlotWindow.PlotWindow(filename, parent=self)
            self.ui.plot_windows[window.name] = window
            print(f'Opening plot window {window.name} from {filename}')

    
    def next_monitor(self):
        monitor = False in self.monitors_used
        if monitor == None:
            return iws_defs.n_monitors
        if monitor < 5:
            self.monitors_used[monitor] = True
            return monitor


    def multi_plot_window_combox_clicked(self, index):
        print(f'CHANGED: {index}')
        if index != 0:
            path = iws_defs.multi_plots_directory
            filename = path + self.ui.multi_plot_window_combobox.itemText(index) + '.json'
            on_monitor = self.next_monitor()
            window = MultiPlotWindow.MultiPlotWindow(filename, on_monitor, parent=self)
            self.ui.plot_windows[window.name] = window
            print(f'Opening multi-plot window {window.name} from {filename}')

    # Open a separate multi plot window
    def open_multi_plot_window_button_clicked(self):
        path = iws_defs.multi_plots_directory
        options = QFileDialog.Options()
        options |= QFileDialog.DontUseNativeDialog
        suffixes = 'json file (*.json)'
        filename, _ = QFileDialog.getOpenFileName(self, "QFileDialog.getOpenFileName()", path, suffixes, options=options)
        
        if filename:
            print(f"Opening multi plot window {filename}")
            window = MultiPlotWindow.MultiPlotWindow(filename, parent=self)
            self.ui.plot_windows[window.name] = window
            print(f'Opening multi-plot window {window.name} from {filename}')


    # ****
    def child_window_closed(self, child):
        print(f'Closing child window {child}')
        try:
            del self.ui.windows[child]
        except Exception as e:
            print(f'Closing child window {child} caused exception {e}')
        pass


    def child_plot_window_closed(self, child):
        print(f'Closing plot window {child}')
        try:
            if hasattr(child, 'on_display'):
                self.monitors_used[child.on_display] = False
            del self.ui.plot_windows[child]
        except Exception as e:
            print(f'Closing plot window {child} caused exception {e}')
        pass

    """
    def open_dialog(self, config_file):
        self.dialog = QtWidgets.QDialog()
        self.dialog.ui = config_dialog()
        self.dialog.ui.setupUi(self.dialog, config_file)

        if self.dialog.exec_():
            #print("Dialog done (OK)")
            #dialog.show()
            pass
        else:
            #print("Dialog done (Cancel)")
            pass
            """

    # Read the path to the .csv file
    def read_ini(self):
        try:
            cfg = configparser.ConfigParser()
            cfg.read(iws_defs.ini_file)
            self.dump_dir = cfg['SOURCE']['dump_dir']
        except Exception as e:
            print(f"Exception '{ex}' thrown in HkMonitorWindow:read_ini")
            pass


    def eof_led(self, onOff):
        #self.ui.eof_led.setStyleSheet("background-color: green") if onOff else self.ui.eof_led.setStyleSheet("background-color: red")
        self.ui.eof_led.setProperty('stat', str(onOff))
        self.ui.eof_led.setStyle(self.ui.eof_led.style())

    """
    def open_source(self):
        # Open config.ini and read source directory
        self.read_ini()

        # Present file open dialog
        options = QFileDialog.Options()
        options |= QFileDialog.DontUseNativeDialog
        suffixes = 'csv file (*.csv)'
        #filename, _ = QFileDialog.getOpenFileName(self, "QFileDialog.getOpenFileName()", self.dump_dir, "All Files (*);;csv file (*.csv);;tsv files (*.tsv)", options=options)
        filename, _ = QFileDialog.getOpenFileName(self, "QFileDialog.getOpenFileName()", self.dump_dir, suffixes, options=options)
        # If file selected open file for monitoring/reading
        if filename:
            self.source = filename
            self.ui.file_textbox.setText(self.source)
            self.source_selected = True
            return 1
        return 
        """

    def directory_changed(self):
        self.dir_changed_count += 1
        print(f'dir changed {self.dir_changed_count}')


    def file_changed(self):
        #sleep(0.05)
        ###self.reader._eof = 0
        #self.reader_out._eof = 0
        if self.ready:
            self.file_changed_count += 1
            #print(f'file changed {self.file_changed_count}')
            self.csv_update()
            #self.csv_update_out()
        else:
            pass
            #self.event_queue.append(True)
            #print('Appending to the event queue')
    '''
    def file_changed(self):
        while self.currently_watching:
            #print('file changed')
            #self.reader._eof = 0
            self.reader._eof = False
            if self.ready:
                self.csv_update()
            self.eof_led(self.reader._eof)
            #if self.reader._eof:
                #self.ui.eof_led.setStyleSheet("background-color: green")
            #else:
                #self.ui.eof_led.setStyleSheet("background-color: red")
            QtWidgets.QApplication.processEvents()
            #sleep(0.05)
            sleep(0.1)
    '''


    def csv_update(self):
        #print('csv_update')
        self.ready = False
        ###self.reader.get_size()
        #while self.reader.data_available() and self.pause_hit == 0:
        while self.reader.data_available() and self.currently_watching:
            #print('csv_update running')
            self.eof_led(self.reader._eof)
            #if self.reader._eof:
                #self.ui.eof_led.setStyleSheet("background-color: green")
            #else:
                #self.ui.eof_led.setStyleSheet("background-color: red")
            result = self.reader.next_line()
            if result != 0:
                print('cvs_update: bad read')
                #return
                ###sleep(0.05) mcrw 20230517
                QtWidgets.QApplication.processEvents()
                continue

            (line_type, string) = self.reader.decode_csv_line()
            #print(f'HkMonitorWindow csv_update: {line_type}, {string}')

            # Handle Event lines
            if line_type == EchoReader.LineType.Event:

                # 2019-11-29T07:25:32.725 Evt: 247/5 TM CCS-SH :   V D G T 2 7 2 0   e n g   1 . 0 3 2 8 3 6   >   h i g h   l i m i t   0 . 8 9 9 9 9
                #obt = string[0]
                s = ''
                for f in string.split(' : '):
                    while '  ' in f:
                        f = f.replace('  ', '@')
                        f = f.replace(' ', '')
                        f = f.replace('@', ' ')
                    s += f

                #self.ui.packet_text.appendPlainText(s)

                #self.event_packet_count += 1
                #string = "Events: %d" % self.event_packet_count
                #self.ui.event_counter_textbox.setText(string)
                QtWidgets.QApplication.processEvents()

            # Handle TC lines
            elif line_type == EchoReader.LineType.TC:
                obt = string[0]
                data = "{:<22} {:<28} {:>3} : {:<25}".format(obt[:22], string[2], string[3], string[1])
                self.ui.event_text.append(data)
                #self.ui.event_text.appendPlainText(data)
                #self.ui.packet_text.appendPlainText(string)
                self.tc_packet_count += 1
                string = "TC Packets: %d" % self.tc_packet_count
                self.ui.tc_packet_counter_textbox.setText(string)
                QtWidgets.QApplication.processEvents()

            # Handle TM parameter lines
            elif line_type == EchoReader.LineType.TMPAR:
                self.ui.update_tm(string)
                obt = string[0]
                if string[3].replace('.', '', 1).isdigit() :
                    self._do_plot = 1
                #if string[3][0].isdigit() :
                    #pass

            # Handle TM HK packet lines
            elif line_type == EchoReader.LineType.HK_PACKET:
                obt = string[0]

                self.ui.update_tab_colour()

                if self._do_plot == 1:
                    self.ui.update_plots(obt, self.time_conv_str)
                else:
                    self._do_plot = 1
                # mcrw 20221012
                get_date_fromobt = datetime.datetime.strptime(obt, self.time_conv_str )

                if self.first_obt == None:
                    self.first_obt = get_date_fromobt

                self.current_obt = get_date_fromobt

                self.ui.obt_textbox.setText('OBT: ' + obt)

                self.hk_packet_count += 1
                pcstring = "Total HK Packets: %d" % self.hk_packet_count
                self.ui.hk_packet_counter_textbox.setText(pcstring)

                sid = string[4]
                #print(f'sid = {sid}')
                if sid == EchoReader.LineType.RT_SLOW:
                    self.rt_slow_count += 1
                    string = "rt slow: %d" % self.rt_slow_count
                    self.ui.rt_slow_packet_counter_textbox.setText(string)
                elif sid == EchoReader.LineType.ST_SLOW:
                    self.st_slow_count += 1
                    string = "st slow: %d" % self.st_slow_count
                    self.ui.st_slow_packet_counter_textbox.setText(string)
                elif sid == EchoReader.LineType.RT_FAST:
                    self.rt_fast_count += 1
                    string = "rt fast: %d" % self.rt_fast_count
                    self.ui.rt_fast_packet_counter_textbox.setText(string)
                elif sid == EchoReader.LineType.RT_SUPERFAST:
                    self.rt_superfast_count += 1
                    string = "rt superfast: %d" % self.rt_superfast_count
                    self.ui.rt_sfast_packet_counter_textbox.setText(string)
                elif sid == EchoReader.LineType.ST_SUPERFAST:
                    self.st_superfast_count += 1
                    string = "st superfast: %d" % self.st_superfast_count
                    self.ui.st_sfast_packet_counter_textbox.setText(string)
                else:
                    pass

                QtWidgets.QApplication.processEvents()

            # Handle TM packet lines
            elif line_type == EchoReader.LineType.TM_PACKET:
#                print(f'TM PACKET: \'{string}\'')
                obt = string[0]

#                if self._do_plot == 1:
#                    self.ui.update_plots(obt, self.time_conv_str)
#                else:
#                    self._do_plot = 1

                # string[1] = 'VIS ST SUFAST HK' (for eg)
                # string[2] = 'TM-HK-PARAM-REPORT (3,25)' (for eg)
                # string[3] = '6' (for eg)
                ###self.ui.packet_text.appendPlainText(obt + ' ' + string[1] + ' ' + string[2] + ' ' + string[3])
                ####self.ui.event_text.appendPlainText(obt + ' ' + string[2] + ' ' + string[3] + ' ' + string[1])

                data = "{:<22} {:<28} {:>3} : {:<25}".format(obt[:22], string[2], string[3], string[1])
                #if '(1,8)' in data:
#                if '(1,7)' in data:
#                    self.ui.event_text.setTextColor(QtGui.QColor.red())
#                else:
#                    self.ui.event_text.setTextColor(QtGui.QColor.black())
                ###self.ui.event_text.appendPlainText(data)
                self.ui.event_text.append(data)

                ###self.ui.event_text.appendPlainText(obt[:22] + ' ' + string[2] + ' ' + string[3] + ' : ' + string[1])

                # mcrw 20221012
                ####get_date_fromobt = datetime.datetime.strptime(obt, "%Y-%m-%dT%H:%M:%S.%f")
                get_date_fromobt = datetime.datetime.strptime(obt, self.time_conv_str )

                if self.first_obt == None:
                    self.first_obt = get_date_fromobt

                self.current_obt = get_date_fromobt

                self.ui.obt_textbox.setText('OBT: ' + obt)

                self.tm_packet_count += 1
                string = "TM Packets: %d" % self.tm_packet_count
                self.ui.tm_packet_counter_textbox.setText(string)

                QtWidgets.QApplication.processEvents()

            elif line_type == EchoReader.LineType.Empty:
                if self._do_plot == 1:
                    pass

                    ###self.ui.update_plots(self.ui.currenttimevalue_label.text())
                    #self.ui.update_plots(str(self.currentobt_done))
                    ###self._do_plot = 0
                else:
                    pass
                QtWidgets.QApplication.processEvents()

            else:
                pass

            ########################QtWidgets.QApplication.processEvents()

        #########sleep(0.05)

        #self.reader._eof = 0
        self.ready = True
        #print(f'cvs_update finished (EOF {self.reader._eof})')
        self.eof_led(self.reader._eof)


    """
    def csv_update_out(self):
        while self.reader_out.data_available():
            result = self.reader_out.next_line()
            if result != 0:
                print('cvs_update_out: bad read')
                return
            (line_type, string) = self.reader_out.decode_csv_line()
            #Peter: line by line read from input file ex1.csv
            #print(string)
            if line_type == EddsReader.LineType.Event:
                #print("EVENT: " + string)
                #self.ui.event_text.appendPlainText(string)
                self.reader_out._ofileId.write(string + '\n')
                pass
            elif line_type == EddsReader.LineType.TC:
                #self.ui.packet_text.appendPlainText(string)
                #self.reader._ofileId.write(string + '\n')
                #print("TC: " + string)
                pass
            elif line_type == EddsReader.LineType.TM:
                #self.ui.update_tm(string)
                self.reader_out._ofileId.write(string + '\n')
            elif line_type == EddsReader.LineType.TM_PACKET:
                #if self._do_plot == 1:
                #    self.ui.update_plots()
                #else:
                #    self._do_plot = 1
                # string is (obt, id, (type,subtype), sid)
#                self.ui.packet_text.appendPlainText(string)
                obt = string[0]
                #self.ui.packet_text.appendPlainText(obt + ' ' + string[1] + ' ' + string[2] + ' ' + string[3])
                self.reader_out._ofileId.write(obt + ',' + string[1] + ','+ string[4] + ',' + string[2] + ',' + string[3] + '\n')
#                self.ui.obt_textbox.setText('OBT: ' + self.reader._logTime)
                #self.ui.obt_textbox.setText('OBT: ' + obt)
                #print("TM: " + string)
            elif line_type == EddsReader.LineType.Empty:
                #if self._do_plot == 1:
                #    self.ui.update_plots()
                #    self._do_plot = 0
                #else:
                #    pass
                pass
            else:
                pass

            QtWidgets.QApplication.processEvents()
        ###sleep(0.05) mcrw 20230517
        self.reader_out._eof = 0
    """


    def start_watching(self):
        self.ready = True
        self.currently_watching = True
        self.reader.open_csv_file(self.source, self.mode)
        #self.reader_out.open_csv_file(self.source, self.mode)
        #self.reader_out.open_to_write(self.source, self.mode)
        self.watcher.addPath(self.source)
        self.watcher.addPath(os.path.dirname(self.source))
        self.watcher.fileChanged.connect(self.file_changed)
        self.watcher.directoryChanged.connect(self.directory_changed)
        watched_files = self.watcher.files()
        for f in watched_files:
            print("Watched file: " + f)
        self.file_changed()


    def stop_watching(self):
        self.ready = False
        self.currently_watching = False
        self.stop_reading()
        self.watcher.removePath(self.source)


    # Connect button pressed
    """
    def top_frame_button_start_clicked(self):
        if self.open_source():
            self.reader.open_csv_file(self.source, self.mode)
            self.reader_out.open_csv_file(self.source, self.mode)
            self.reader_out.open_to_write(self.source, self.mode)
            self.watcher = QtCore.QFileSystemWatcher([self.source])
            self.watcher.fileChanged.connect(self.file_changed)
            watched_files = self.watcher.files()
            #for f in watched_files:
                #print("Watched file: " + f)
            self.ui.top_frame_button_stop.setEnabled(True)
            self.ui.top_frame_button_start.setEnabled(False)
            """
            
    # Stop button pressed
    def top_frame_button_stop_clicked(self):
        self.stop_reading()


    def stop_reading(self):
        self.reader.stop()
        #self.reader_out.stop()
        self.reader.close_input()
        #self.reader_out.close_input()
        #self.ui.top_frame_button_stop.setEnabled(False)
        #self.ui.top_frame_button_start.setEnabled(True)


    # Quit button clicked
    #def top_frame_button_quit_clicked(self):
    def quit_button_clicked(self):
        self.doQuit1()


    # Not used any more
    def doQuit(self):
        self.stop_reading()

        quitbuttonReply = QMessageBox.question(self,
                                               'PyQt5 message', "Are you sure you want to quit the HK/TM monitor?",
                                               QMessageBox.Yes | QMessageBox.No, QMessageBox.No)

        if quitbuttonReply == QMessageBox.Yes:
            if self.source_selected:
                # Write back to config.ini file
                try:
                    cfg = configparser.ConfigParser()
                    path = os.path.dirname(self.source)
                    print("Saving " + path + " to ini file at " + iws_defs.ini_file)
                    cfg['SOURCE'] = {'dump_dir' : path}
                    with open(iws_defs.ini_file, 'w') as ini_file:
                        cfg.write(ini_file)
                except  Exception as e:
                    print('Failed to write to ini file at ' + iws_defs.ini_file)
                    print(e)
                self.close()
            ###exit(-2)
        else:
            #print('No clicked.')
            pass

        return


    def doQuit1(self):
        self.stop_watching()
        for window in self.ui.windows.values():
            #if self.ui.windows[window.name] != None:
                #del self.ui.windows[window.name]
            window.close()
        for window in self.ui.plot_windows.values():
            #if self.ui.plot_windows[window.name] != None:
                #del self.ui.plot_windows[window.name]
            window.close()
            
        closedOk = True
        if self.source_selected:
            # Write back to config.ini file
            try:
                cfg = configparser.ConfigParser()
                path = os.path.dirname(self.source)
                print("Saving " + path + " to ini file at " + iws_defs.ini_file)
                cfg['SOURCE'] = {'dump_dir' : path}
                with open(iws_defs.ini_file, 'w') as ini_file:
                    cfg.write(ini_file)
            except Exception as e:
                print('Failed to write to ini file at ' + iws_defs.ini_file)
                print(e)
                closedOk = False
        self.close()
        return closedOk



if __name__ == '__main__':

    # Handle high resolution displays:                                                                                  
    if hasattr(QtCore.Qt, 'AA_EnableHighDpiScaling'):
        print("Enabling high dpi scaling")
        QApplication.setAttribute(QtCore.Qt.AA_EnableHighDpiScaling, True)
    if hasattr(QtCore.Qt, 'AA_UseHighDpiPixmaps'):
        print("Using high dpi pixmaps")
        QApplication.setAttribute(QtCore.Qt.AA_UseHighDpiPixmaps, True)

    #Peter: pdb start here
    #import pdb; pdb.set_trace()
    app = QApplication(sys.argv)
    #app.setStyleSheet("QGroupBox {background-color:pink;}")

    # Stolen from Themis' code...
    branch = os.popen('git rev-parse --abbrev-ref HEAD').read()[:-1]
    #####version = os.popen("git describe").read()[:-1]
    conf_type="CEA_ORIG"

    #print(str(len(sys.argv)))

#    if len(sys.argv) > 1 :
#        conf_type = sys.argv[1]

#    window = HkMonitorWindow("Mock IWS CC (%s)" % (branch), conf_type)

    with open(iws_defs.config_yaml_file, 'r') as file:
        yaml_config = yaml.safe_load(file)

    #s = HkMonitorWindow.HkMonitorWindow(config=os.path.basename(self.configuration), source=self.source, config_type=self.configType)
    window = HkMonitorWindow(main=True, config=yaml_config['config'], source=yaml_config['source'], config_type=yaml_config['protocol'])


    screen = app.primaryScreen()
    print('Screen: %s' % screen.name())
    size = screen.size()
    print('Screen Size: %d x %d' % (size.width(), size.height()))
    rect = screen.availableGeometry()
    print('Screen Available Geometry: %d x %d' % (rect.width(), rect.height()))

    #   Scroll Area Propertieskk
    #scroll = QScrollArea()
    #scroll.setFrameShape(frame)
    #scroll.setVerticalScrollBarPolicy(Qt.ScrollBarAlwaysOn)
    #scroll.setHorizontalScrollBarPolicy(Qt.ScrollBarAlwaysOff)
    #scroll.setWidgetResizable(True)
    #scroll.setWidget(window)

    #scroll_layout = QVBoxLayout()
    #scroll_layout.addWidget(scroll)
    #scroll_layout.setContentsMargins(0, 0, 0, 0)
    #self.setLayout(scroll_layout) 
    #window.setLayout(scroll_layout) 
    window.show()

    sys.exit(app.exec_())
