""" This class creates and laysout a page of HK parameters to be displayed in a tab.


"""

import sys
import re

from PyQt5 import QtWidgets, QtCore, QtGui      #, uic, QtQml, QtQuick
from PyQt5.QtCore import Qt

#from Euclid import iws_defs, iws, DisplayBox, EuclidHkBox
import iws_defs, iws, DisplayBox, EuclidHkBox

""" These constants refer to the offset of strings in the input feed from the data file """
MIB_NAME = 0
MIB_ID   = 1
HK_UNIT  = 2

fpn = re.compile('[+-]?(\d+(\.\d*)?|\.\d+)([eE][+-]?\d+)?')

class LabelsPage(QtWidgets.QFrame):

    #def __init__(self, parent_frame, label_file):
    def __init__(self, parent_frame, json_data):
        """ json_data = {name:name,labels:[{mib_name:mn,mib_id:mi,source:src,unit:v} """
        """ json_data = {name:n,groups:[{row:r,column:c,labels:[{mib_name:mn,mib_id:mi,source:src,unit:v}]}]} """
        super(LabelsPage, self).__init__()
        self.labels = []
        self.label_number = 0
        #self.labels = iws.get_alt_json_labels(json_data)
        self.parameters = {}
        self.raw = False
        #self.debug()
        self.setupUi(json_data)
        #self.debug()

    def debug(self):
        """ Display all the labels """
        for line in self.labels:
            #print("label: %s %s %s %s" % (line[0], line[1], line[2], line[3]))
            pass


    def debug_parameters(self):
        """ Displays all te parameters """
        print("debug_parameters:")
        for line in self.labels:
            mib = line[1]
            print(mib, self.parameters[mib][0].text())
            #print(mib, self.parameters[mib])


    def rawOnClicked(self):
        """ Toggle between showing engineering values and raw values. """
        if self.checkbox.isChecked():
            #print("checkbox toggled on")
            self.raw = False
            for x in range(0, len(self.parameters)):
                mib_id = self.labels[x][MIB_ID]
                self.parameters[mib_id].value_box.setText(self.parameters[mib_id].eng_string)
        else:
            #print("checkbox toggled off")
            self.raw = True
            for x in range(0, len(self.parameters)):
                mib_id = self.labels[x][MIB_ID]
                self.parameters[mib_id].value_box.setText(self.parameters[mib_id].raw_string)


    def mibOnClicked(self):
        """ Toggle between displaying parameter MIB names or MIB ids. """
        label_index = 0
        if self.mib_checkbox.isChecked():
            #print("mib_checkbox toggled on")
            pass
        else:
            #print("mib_checkbox toggled off")
            label_index = 1
        for x in range(0, len(self.parameters)):
            mib_id = self.labels[x][MIB_ID]
            #self.parameters[mib_id][MIB_NAME].setText(self.labels[x][label_index])
            self.parameters[mib_id].label_box.setText(self.labels[x][label_index])


    # fields:
    # 'logTime', 'mib_id', 'rawVal', 'engVal', 'monState'
    def update_tm(self, fields):
        """ Update the parameter value box with a new value read from file. """
        try:
            mib_id = fields[1]
            #plabel = self.parameters[mib_id][MIB_NAME]
            #vlabel = self.parameters[mib_id][MIB_ID]
            #unit   = self.parameters[mib_id][HK_UNIT]
            plabel = self.parameters[mib_id].label_box
            vlabel = self.parameters[mib_id].value_box
            unit   = self.parameters[mib_id].unit_string
            if unit == '-':
                unit = ' '
            rawVal = fields[2]
            engVal = fields[3]
            status = fields[4]

            #print('RAW = ' + self.parameters[mib_id][3])
            #self.parameters[mib_id][3] = rawVal
            self.parameters[mib_id].raw_string = rawVal
            if self.raw:
                display_string = rawVal
            else:
                if fpn.match(engVal):
                    #display_string = "MATCH!" + engVal
                    val = "%3.03f" % float(engVal)     # %3.03f ?
                    #print ("val = " + str(val))
                    #print("DOFJOIDFIODFHIODFJ")
                    display_string = str(val) + " " + unit
                else:
                    #display_string = "BOOOO!"
                    display_string = engVal + " " + unit
            #self.parameters[mib_id][4] = display_string
            self.parameters[mib_id].eng_string = display_string

            #display_string = rawVal if self.raw else engVal + " " + unit
 
            #if (status == "ALARM") or (engVal == "*NO VAL*"):
            if status == "ALARM":
                newProp = 'ool'
                #plabel.setProperty('stat', 'ool')
                #vlabel.setProperty('stat', newProp)
            elif engVal == "*NO VAL*":
                newProp = 'warn'
            else:
                newProp = 'ok'

                #s = plabel.style()
                #plabel.setStyle(s)
                #vlabel.setStyle(s)
            plabel.setProperty('stat', newProp)
            vlabel.setProperty('stat', newProp)
                
            plabel.setStyle(plabel.style())
            vlabel.setStyle(vlabel.style())

            vlabel.setText(display_string)
            ##vlabel.setText(engVal + " " + unit + " ")
            #vlabel.setText(engVal)

            #print("TM received " + fields[0] + " " + mib_id + " " + engVal + " [" + status + "]")

        except Exception as e:
            #print(mib_id + " ?")
            pass

            
# Labels: VIS_CDPU_5V_CPU VDGT2712 rt_slow unit
    def setupUi(self, json_data):
        """ Read in the parameters and layout the boxes. """

        #####self.setStyleSheet("background-color:lightblue;")

        self.checkbox = QtWidgets.QCheckBox("Engineering", self)
        self.checkbox.setObjectName("checkbox")
        self.checkbox.setChecked(True)
        self.checkbox.setGeometry(5, 5, 100, 16)
        self.checkbox.toggled.connect(self.rawOnClicked)

        self.mib_checkbox = QtWidgets.QCheckBox("Use MIB name", self)
        self.mib_checkbox.setObjectName("mib_checkbox")
        self.mib_checkbox.setChecked(True)
        self.mib_checkbox.setGeometry(110, 5, 115, 16)
        self.mib_checkbox.toggled.connect(self.mibOnClicked)

        for group in json_data['groups']:
            grid_row = group['row']
            grid_column = group['column']
            labels = group['labels']
            label_row = grid_row * 12
            for label in labels:
                mib_name = label['mib_name']
                mib_id   = label['mib_id']
                source   = label['source']
                unit     = label['unit']
                #self.labels.append((mib_name, mib_id, label['source'], label['unit']))
                self.labels.append((mib_name, mib_id, source, unit))
                (label_box, value_box) = DisplayBox.make_box(self, grid_column, label_row, mib_name)
                label_box.setToolTip(mib_id)
                label_box.setToolTipDuration(500)
#                print(f"Doing {mib_id}")
                label_row += 1
                self.parameters[mib_id] = EuclidHkBox.EuclidHkBox(label_box, value_box, self.labels[self.label_number][3])
                self.label_number += 1

        """
        # Make labels
        column = 0
        row    = 0
        for label_number in range(0, len(self.labels)):
            mib_name = self.labels[label_number][MIB_NAME]
            #print("mib_name: %s" % mib_name)
            mib_id = self.labels[label_number][MIB_ID]
            #print("mib_id: %s" % mib_id)
            (label_box, value_box) = DisplayBox.make_box(self, column, row, mib_name)
            row += 1
            #if (label_number + 1) % 21 == 0:
            if (label_number + 1) % 24 == 0:
                column += 1
                row = 0

            # Each parameter, identified by its mib id, has a label box, a value box and a unit string
            # Update to have a raw string and an engineering string
            #self.parameters[mib_id] = (label_box, value_box, self.labels[label_number][3])

            #self.parameters[mib_id] = (label_box, value_box, self.labels[label_number][3], value_box.text(), value_box.text())
            self.parameters[mib_id] = EuclidHkBox.EuclidHkBox(label_box, value_box, self.labels[label_number][3])

        #self.debug_parameters()
        """
