#!/usr/bin/env python                                                                                                                             
import sys
import os
import time
from time import sleep

from PyQt5 import QtWidgets, QtGui, QtCore    #, uic, QtQml, QtQuick
from PyQt5.QtWidgets import QFileDialog, QMessageBox
from PyQt5.QtCore import Qt, QTimer

from Dialog import Ui_Dialog as config_dialog

import configparser

import EchoReader
import iws_defs

import HkMonitorUi
import TabsWindow
import PlotWindow

class HkMonitorWindow(QtWidgets.QMainWindow):
    
    #def __init__(self, *args, **kargs):
    def __init__(self, *args, config='/foo', data_source='~/Desktop/ex1.csv'):
        super(HkMonitorWindow, self).__init__()

        print("Called with " + str(len(args)) + " args")

        self.setObjectName("MainWindow")
        #self.resize(550, 550)
        self.setWindowTitle("IWS TMHK Monitor")

        #self.watcher = None
        self.watcher = QtCore.QFileSystemWatcher()

        #self.source = os.environ['HOME'] + "/Projects/Euclid/DART_dumps/CEA_Day3_3.25.csv"
        self.source = '' #os.environ['HOME'] + "/Projects/Euclid/DART_dumps/CEA_Day3_3.25.csv"
        self.source_selected = False

        self.file_format = ''
        self.dump_dir = os.environ['HOME']

        self.reader = EchoReader.EchoReader()

        self.output_lines = 0

        self._do_plot = 0

        #self.ui = HkMonitorUi.HkMonitorUi(self)
        #self.setCentralWidget(self.ui)

        #self.show()

        #config='/foo', data_source='~/Desktop/ex1.csv')

        self.source_selected = True
        self.source = os.path.expanduser(data_source)
        #self.read_ini()
        #config = args[0]
        self.layout(iws_defs.defaults_directory + config)
        self.start_watching()
    
        """
        if len(args) == 2:
            self.source_selected = True
            self.source = os.path.expanduser(args[1])
            #self.read_ini()
            config = args[0]
            self.layout(iws_defs.defaults_directory + config)
            self.start_watching()

        else:
            self.open_dialog(iws_defs.config_file)
            config = iws_defs.config
            self.layout(iws_defs.defaults_directory + config)
            """

    def closeWindow(self):
        print("closeWindow")
        self.doQuit1()
        pass

    # Create the ui
    def layout(self, config_file):
        print("Layout")
        self.ui = HkMonitorUi.HkMonitorUi(self)

        self.ui.layout(self, config_file)

        self.setCentralWidget(self.ui)

        #self.ui.top_frame_button_start.clicked.connect(self.top_frame_button_start_clicked)
        #self.ui.top_frame_button_stop.clicked.connect(self.top_frame_button_stop_clicked)
        #self.ui.top_frame_button_quit.clicked.connect(self.top_frame_button_quit_clicked)
        self.ui.bottom_frame_button.clicked.connect(self.bottom_frame_button_clicked)
        self.ui.bottom_frame_button1.clicked.connect(self.bottom_frame_button1_clicked)

        self.ui.file_textbox.setText("Reading: " + self.source)

        self.show()


    def bottom_frame_button_clicked(self):
        path = iws_defs.windows_directory
        options = QFileDialog.Options()
        options |= QFileDialog.DontUseNativeDialog
        suffixes = 'json file (*.json)'
        filename, _ = QFileDialog.getOpenFileName(self, "QFileDialog.getOpenFileName()", path, suffixes, options=options)
        
        if filename:
            #print(f"Opening window {filename}")
            window = TabsWindow.TabsWindow(filename, parent=self)
            #print('Appending window')
            self.ui.windows[window.name] = window


    def bottom_frame_button1_clicked(self):
        path = iws_defs.plots_directory
        options = QFileDialog.Options()
        options |= QFileDialog.DontUseNativeDialog
        suffixes = 'json file (*.json)'
        filename, _ = QFileDialog.getOpenFileName(self, "QFileDialog.getOpenFileName()", path, suffixes, options=options)
        
        if filename:
            #print(f"Opening window {filename}")
            window = PlotWindow.PlotWindow(filename, parent=self)
            #print('Appending window')
            self.ui.plot_windows[window.name] = window


    def child_window_closed(self, child):
        del self.ui.windows[child]
        pass

    def child_plot_window_closed(self, child):
        del self.ui.plot_windows[child]
        pass


    def open_dialog(self, config_file):
        self.dialog = QtWidgets.QDialog()
        self.dialog.ui = config_dialog()
        self.dialog.ui.setupUi(self.dialog, config_file)

        if self.dialog.exec_():
            #print("Dialog done (OK)")
            #dialog.show()
            pass
        else:
            #print("Dialog done (Cancel)")
            pass


    # Read the path to the .csv file
    def read_ini(self):
        try:
            cfg = configparser.ConfigParser()
            cfg.read(iws_defs.ini_file)
            self.dump_dir = cfg['SOURCE']['dump_dir']
        except Exception as e:
            pass


    def open_source(self):
        # Open config.ini and read source directory
        self.read_ini()

        # Present file open dialog
        options = QFileDialog.Options()
        options |= QFileDialog.DontUseNativeDialog
        suffixes = 'csv file (*.csv)'
        #filename, _ = QFileDialog.getOpenFileName(self, "QFileDialog.getOpenFileName()", self.dump_dir, "All Files (*);;csv file (*.csv);;tsv files (*.tsv)", options=options)
        filename, _ = QFileDialog.getOpenFileName(self, "QFileDialog.getOpenFileName()", self.dump_dir, suffixes, options=options)
        # If file selected open file for monitoring/reading
        if filename:
            self.source = filename
            self.ui.file_textbox.setText(self.source)
            self.source_selected = True
            return 1
        return 

    def file_changed(self):
        sleep(0.05)
        self.reader._eof = 0
        self.csv_update()


    def csv_update(self):
        while self.reader.data_available():
            result = self.reader.next_line()
            if result != 0:
                print('cvs_update: bad read')
                return
            (line_type, string) = self.reader.decode_csv_line()
            if line_type == EchoReader.LineType.Event:
                #print("EVENT: " + string)
                self.ui.event_text.appendPlainText(string)
                pass
            elif line_type == EchoReader.LineType.TC:
                self.ui.packet_text.appendPlainText(string)
                #print("TC: " + string)
                pass
            elif line_type == EchoReader.LineType.TM:
                self.ui.update_tm(string)
            elif line_type == EchoReader.LineType.TM_PACKET:
                if self._do_plot == 1:
                    self.ui.update_plots()
                else:
                    self._do_plot = 1
                self.ui.packet_text.appendPlainText(string)
                self.ui.obt_textbox.setText('OBT: ' + self.reader._logTime)
                #print("TM: " + string)
            elif line_type == EchoReader.LineType.Empty:
                if self._do_plot == 1:
                    self.ui.update_plots()
                    self._do_plot = 0
                else:
                    pass
            else:
                pass

            QtWidgets.QApplication.processEvents()
        sleep(0.05)
        self.reader._eof = 0


    def start_watching(self):
        self.reader.open_csv_file(self.source)
        self.watcher.addPath(self.source)
        self.watcher.fileChanged.connect(self.file_changed)
        watched_files = self.watcher.files()
        for f in watched_files:
            print("Watched file: " + f)
        #self.ui.top_frame_button_stop.setEnabled(True)
        #self.ui.top_frame_button_start.setEnabled(False)


    # Connect button pressed
    def top_frame_button_start_clicked(self):
        if self.open_source():
            self.reader.open_csv_file(self.source)
            self.watcher = QtCore.QFileSystemWatcher([self.source])
            self.watcher.fileChanged.connect(self.file_changed)
            watched_files = self.watcher.files()
            for f in watched_files:
                print("Watched file: " + f)
            self.ui.top_frame_button_stop.setEnabled(True)
            self.ui.top_frame_button_start.setEnabled(False)


    # Stop button pressed
    def top_frame_button_stop_clicked(self):
        self.stop_reading()
        self.ui.top_frame_button_stop.setEnabled(False)
        self.ui.top_frame_button_start.setEnabled(True)


    def stop_reading(self):
        self.reader.stop()
        self.reader.close_input()
        #self.ui.top_frame_button_stop.setEnabled(False)
        #self.ui.top_frame_button_start.setEnabled(True)


    # Quit button clicked
    def top_frame_button_quit_clicked(self):
        quitbuttonReply = QMessageBox.question(self,
                                               'PyQt5 message', "Are you sure you want to quit the HK/TM monitor?",
                                               QMessageBox.Yes | QMessageBox.No, QMessageBox.No)
        if quitbuttonReply == QMessageBox.Yes:
            if self.source_selected:
                # Write back to config.ini file                                                                                                 
                try:
                    cfg = configparser.ConfigParser()
                    path = os.path.dirname(self.source)
                    print("Saving " + path + " to ini file at " + iws_defs.ini_file)
                    cfg['SOURCE'] = {'dump_dir' : path}
                    with open(iws_defs.ini_file, 'w') as ini_file:
                        cfg.write(ini_file)
                except  Exception as e:
                    print('Failed to write to ini file at ' + iws_defs.ini_file)
                    print(e)
                self.close()
            exit(-2)
        else:
            #print('No clicked.')
            pass

        return


    def doQuit1(self):
        self.stop_reading()
        print("CLOSING")
        for window in self.ui.windows.values():
            print(f"Closing {window.name}")
            if self.ui.windows[window.name] != None:
                #self.ui.windows[window.name].close()
                #self.ui.windows[window.name] = None
                del self.ui.windows[window.name]
                window.close()
        for window in self.ui.plot_windows.values():
            print(f"Closing {window.name}")
            if self.ui.plot_windows[window.name] != None:
                #self.ui.windows[window.name].close()
                #self.ui.plot_windows[window.name] = None
                del self.ui.plot_windows[window.name]
                window.close()
            

        closedOk = True
        if self.source_selected:
            # Write back to config.ini file
            try:
                cfg = configparser.ConfigParser()
                path = os.path.dirname(self.source)
                print("QUIT: Saving " + path + " to ini file at " + iws_defs.ini_file)
                cfg['SOURCE'] = {'dump_dir' : path}
                with open(iws_defs.ini_file, 'w') as ini_file:
                    cfg.write(ini_file)
            except Exception as e:
                print('Failed to write to ini file at ' + iws_defs.ini_file)
                print(e)
                closedOk = False
        self.close()
        return closedOk


if __name__ == '__main__':
    app = QtWidgets.QApplication(sys.argv)
    #app.setStyleSheet("QGroupBox {background-color:pink;}")
    window = HkMonitorWindow()
    ##################window.open_dialog(iws_defs.config_file)
    ##################config = iws_defs.config
    ##################window.layout(iws_defs.defaults_directory + config)

    # Done in window.layout - can do it here instead
    ### window.ui.top_frame_button_start.clicked.connect(window.top_frame_button_start_clicked)

    #window.ui.debug()

    sys.exit(app.exec_())
