from PyQt5 import QtCore, QtWidgets, QtGui
from PyQt5.QtCore import Qt, QMimeData, QRegExp
from PyQt5.QtWidgets import QTreeWidgetItem, QFrame, QFileDialog, QMessageBox
from PyQt5.QtGui import QDrag, QFont, QRegExpValidator

import json

import make_config_defs

import List
import ListItem

"""
This class mimics the plot frame in the HKTM monitor and is used to define a new plot window for the monitor.
"""
class PlotFrame(QtWidgets.QFrame):
    def __init__(self, parent=None):
        super(PlotFrame, self).__init__(parent)

        self.parent = parent

        self.combo_index = 0

        master_frame_layout = QtWidgets.QVBoxLayout()

        #label_frame = self.make_label_frame()

        label_frame = QtWidgets.QFrame()
        #label_frame.setStyleSheet("background-color:pink;")
        label_frame_layout = QtWidgets.QHBoxLayout()

        page_name_label = QtWidgets.QLabel("Plot Name:")
        self.page_name_box = QtWidgets.QLineEdit()
        rx = QRegExp("[A-Za-z]+[A-Za-z0-9_]*")
        v = QRegExpValidator(rx, self)
        self.page_name_box.setValidator(v)
        self.page_name_box.editingFinished.connect(self.nameEdited)
        self.page_name_box.textChanged.connect(self.nameEdited)

        unit_label = QtWidgets.QLabel("Unit:")
        self.combo = QtWidgets.QComboBox()
        self.combo.addItem("mV")
        self.combo.addItem("Volts")
        self.combo.addItem("mA")
        self.combo.addItem("Amps")
        self.combo.addItem("Temperature (C)")
        self.combo.addItem("Temperature (K)")
        self.combo.addItem("None")
        self.combo.activated.connect(self.handleActivated)

        min_label = QtWidgets.QLabel("Min:")
        max_label = QtWidgets.QLabel("Max:")
        self.min_box = QtWidgets.QLineEdit()
        self.max_box = QtWidgets.QLineEdit()
        
        label_frame_layout.addWidget(page_name_label)
        label_frame_layout.addWidget(self.page_name_box)
        label_frame_layout.addWidget(unit_label)
        label_frame_layout.addWidget(self.combo)
        label_frame_layout.addWidget(min_label)
        label_frame_layout.addWidget(self.min_box)
        label_frame_layout.addWidget(max_label)
        label_frame_layout.addWidget(self.max_box)

        label_frame_layout.addStretch(1)
        label_frame.setLayout(label_frame_layout)

        page_frame = QtWidgets.QFrame()
        #page_frame.setStyleSheet("background-color:yellow;")
        page_layout = QtWidgets.QGridLayout()

        self.page_listboxes = []
        for i in range(12):
            name = "Box_%d" % (i)
            l = List.List(name,parent=self)
            sp_retain = l.sizePolicy()
            sp_retain.setRetainSizeWhenHidden(True)
            row = 0 if i < 6 else  1
            col = i if i < 6 else i - 6
            if row == 0 and col == 0:
                l.setVisible(True)
            else:
                l.setVisible(False)
            l.setSizePolicy(sp_retain)
            self.page_listboxes.append(l)
            page_layout.addWidget(self.page_listboxes[i], row, col)    
        self.list = self.page_listboxes[0]
        page_frame.setLayout(page_layout)

        bottom_frame = QtWidgets.QFrame()
        #bottom_frame.setStyleSheet("background-color:green;")
        bottom_frame_layout = QtWidgets.QHBoxLayout()
        self.clear_button = QtWidgets.QPushButton("Clear")
        self.add_button = QtWidgets.QPushButton("Add")
        self.add_button.setEnabled(False)
        self.save_button = QtWidgets.QPushButton("Save")
        self.save_button.setEnabled(False)
        self.read_button = QtWidgets.QPushButton("Read")

        self.add_button.clicked.connect(self.add_pressed)
        self.clear_button.clicked.connect(self.clear)
        self.save_button.clicked.connect(self.save)
        self.read_button.clicked.connect(self.read)

        bottom_frame_layout.addWidget(self.clear_button)
        bottom_frame_layout.addWidget(self.add_button)
        bottom_frame_layout.addStretch(1)
        bottom_frame_layout.addWidget(self.read_button)
        bottom_frame_layout.addWidget(self.save_button)
        bottom_frame.setLayout(bottom_frame_layout)

        master_frame_layout.addWidget(label_frame)
        master_frame_layout.addWidget(page_frame)
        master_frame_layout.addWidget(bottom_frame)

        self.setLayout(master_frame_layout)


    def make_label_frame(self):
        frame  = QtWidgets.QFrame()
        layout = QtWidgets.QHBoxLayout()

        return frame


    def handleActivated(self, index):
        self.combo_index = index


    def clear(self):
        self.page_name_box.setText('')
        self.min_box.setText('')
        self.max_box.setText('')
        nitems = self.list.count()
        for n in range(nitems):
            #print(f"Removing item {n}")
            self.list.takeItem(0)

    def nameEdited(self):
        name = self.page_name_box.text()
        enable = True if name != '' else False
        self.save_button.setEnabled(enable)
        self.add_button.setEnabled(enable)
        pass


    def add_pressed(self):
        name = self.page_name_box.text()
        if name == '':
            self.parent.statusbar.showMessage("Plot name is required")
            return False
        unit = self.combo.itemText(self.combo_index)
        minv = self.min_box.text()
        if minv == '':
            self.parent.statusbar.showMessage("Minimum value is required")
            return False
        maxv = self.max_box.text()
        if maxv == '':
            self.parent.statusbar.showMessage("Maximum value is required")
            return False
        if self.list.count() == 0:
            self.parent.statusbar.showMessage("At least one plot parameter is required")
            return False
        ret = self.parent.add_plot(name, self.list, minv, maxv, unit)
        if ret:
            self.clear()
            return ret


    def save(self):
        name = self.page_name_box.text()
        unit = self.combo.itemText(self.combo_index)
        minv = self.min_box.text()
        maxv = self.max_box.text()
        min_value = 0.0
        max_value = 0.0
        try:
            min_value = float(minv)
            max_value = float(maxv)
        except Exception as e:
            print(f"An error occurred converting minv {minv} or maxv {maxv}")
        # Check name is set
        if self.list.count() == 0:
            self.parent.statusbar.showMessage("At least one parameter is required")
        else:
            dict = {}
            group = []
            group.append(self.parent.make_group(self.list, 0))
            dict['name'] = name
            dict['naxes'] = 1
            yaxis1 = {}
            yaxis1['label'] = unit
            yaxis1['min'] = min_value
            yaxis1['max'] = max_value
            dict['yaxis1'] = yaxis1
            dict['yaxis2'] = {}
            dict['labels'] = group
            
            options = QFileDialog.Options()
            options |= QFileDialog.DontUseNativeDialog
            suffixes = 'json file (*.json)'

            filename, _ = QFileDialog.getSaveFileName(self, "QFileDialog.getSaveFileName()", make_config_defs.plot_tabs_dir, suffixes, options=options)
            if filename:
                print(f"filename is {filename}")
                if filename.endswith('.json'):
                    print(f"(has json) filename = {filename}")
                    pass
                else:
                    filename += '.json'
                    print(f"(needs json) filename = {filename}")
                with open(filename, 'w') as outfile:
                    json.dump(dict, outfile)
        pass


    def read(self):
        options = QFileDialog.Options()
        options |= QFileDialog.DontUseNativeDialog
        suffixes = 'json file (*.json)'
        filename, _ = QFileDialog.getOpenFileName(self, "QFileDialog.getOpenFileName()", make_config_defs.plot_tabs_dir, suffixes, options=options)
        if filename:
            self.clear()
            with open(filename) as f:
                data = json.load(f)
                self.page_name_box.setText(data['name'])
                yaxis = data['yaxis1']
                self.min_box.setText(str(yaxis['min']))
                self.max_box.setText(str(yaxis['max']))
                unit = yaxis['label']
                index = self.combo.findText(unit)
                if index == -1:
                    print("ARRRGHHH!")
                else:
                    self.combo.setCurrentIndex(index)
                labels = data['labels']
                group = labels[0] # [{'row':0,..., 'labels':[]}]
                labels = group['labels']
                for label in labels:
                    item = ListItem.ListItem(label['mib_name'], label['mib_id'], label['source'], label['unit'], 0)
                    self.list.addItem(item)
        pass
